package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.AccountSecurity;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.entity.TenantAccount;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.interfaces.TrxService;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.LoginModel;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.bocloud.sms.service.internel.CosInternelService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.LockFactory;
import com.megatron.framework.utils.IpTool;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 登录控制器
 *
 * @author dmw
 */
@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping("/v1")
@Tag(name = "登录管理")
public class LoginController {

    private final UserService userService;
    private final TenantService tenantService;
    private final CosInternelService cosInternelService;
    private final SystemConfigRepository systemConfigRepository;
    private final StringRedisTemplate redisTemplate;
    private final LockFactory lockFactory;
    private final TrxService trxService;

    /**
     * 用户登录
     *
     * @param loginModel 登录信息
     * @param request    浏览器请求
     * @return 登录结果
     */
    @Operation(summary = "用户登录")
    @PostMapping("/users/login")
    public GeneralResult<Map<String, Object>> userLogin(@RequestBody LoginModel loginModel,
                                                        HttpServletRequest request) {
        try {
            log.info("开始执行登录请求");
            long start;
            String account = loginModel.getAccount();
            String password = loginModel.getPassword();
            log.info("开始调用UserService");
            GeneralResult<Map<String, Object>> loginResult = userService
                    .login(account, password, request.getSession().getId(), IpTool.getIP(request), loginModel.getIsManager());
            log.info("请求方ip地址trx：" + trxService.getRequestIpAddress(request));
            log.info("请求方ip地址：" + IpTool.getIP(request));
            log.info("结束调用UserService");
            if (loginResult.isFailed()) {
                if (null == loginResult.getData()) {
                    String path = "Login_" + account;
                    try (com.megatron.framework.lock.AutoCloseLock lock = lockFactory.getACLock(path)) {
                        Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                        start = System.currentTimeMillis();
                        String value = redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + account);
                        log.info("查询Redis耗时{}ms", System.currentTimeMillis() - start);
                        SystemConfig errorNumber = systemConfigRepository.queryByCode("pwdErrorNumber");
                        int error = Integer.parseInt(errorNumber.getValue());
                        if (!StringUtils.hasText(value)) {
                            //密码操作失败
                            start = System.currentTimeMillis();
                            redisTemplate.opsForValue()
                                    .set(RequestContext.Catalog.User.name() + account, String.valueOf(1), 60, TimeUnit.SECONDS);
                            log.info("写入Redis耗时{}ms", System.currentTimeMillis() - start);
                            loginResult = new GeneralResult<>(false, loginResult.getMessage() + "，已操作【1】次");
                        } else if (Integer.parseInt(value) >= error) {
                            Result lockResult = userService.lockByAccount(account);
                            boolean success = lockResult.isSuccess();
                            start = System.currentTimeMillis();
                            redisTemplate.delete(RequestContext.Catalog.User.name() + account);
                            log.info("删除Redis耗时{}ms", System.currentTimeMillis() - start);
                            if (success) {
                                loginResult = new GeneralResult<>(false, "账号或者密码错误已达【" + error + "】次,若用户存在将被冻结，请联系管理员解冻");
                            } else {
                                loginResult = new GeneralResult<>(false, "认证服务账号缓存存在问题");
                            }
                        } else {
                            start = System.currentTimeMillis();
                            redisTemplate.boundValueOps(RequestContext.Catalog.User.name() + account).increment(1L);
                            log.info("写入Redis耗时{}ms", System.currentTimeMillis() - start);
                            String num = redisTemplate.opsForValue().get(RequestContext.Catalog.User.name() + account);
                            loginResult = new GeneralResult<>(false, loginResult.getMessage() + "，已操作【" + num + "】次");
                        }
                        log.info("登录请求处理完成");
                        return loginResult;
                    } catch (Exception e) {
                        log.error("Get login error message：", e);
                        log.info("登录请求处理完成");
                        return loginResult;
                    }
                }
            }
            start = System.currentTimeMillis();
            redisTemplate.delete(RequestContext.Catalog.User.name() + account);
            log.info("删除Redis耗时{}ms", System.currentTimeMillis() - start);
            log.info("登录请求处理完成");
            return loginResult;
        }catch (Exception e){
            log.error("登录异常",e);
            return new GeneralResult<>(false,"登录异常:" + e.getMessage());
        }
    }

    /**
     * 用户登出
     *
     * @param requestContext 上下文
     * @return 登出结果
     */
    @Operation(summary = "用户退出系统")
    @PostMapping("/users/logout")
    public Result userLogout(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.logout(requestContext.getTarget());
    }


    /**
     * 租户登录
     *
     * @param loginModel 租户登录信息
     * @param request    浏览器请求
     * @return 登录结果，token
     */
    @Operation(summary = "租户登录")
    @PostMapping("/tenants/login")
    public GeneralResult<Map<String, Object>> tenantLogin(
            @RequestBody LoginModel loginModel, HttpServletRequest request) {
        String account = loginModel.getAccount();
        String password = loginModel.getPassword();
        GeneralResult<Map<String, Object>> loginResult =
                tenantService.login(account, password, request.getSession().getId(), IpTool.getIP(request));
        if (loginResult.isFailed()) {
            if (null == loginResult.getData()) {
                String path = "Login_" + account;
                try (com.megatron.framework.lock.AutoCloseLock lock = lockFactory.getACLock(path)) {
                    Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                    String value = redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + account);
                    SystemConfig errorNumber = systemConfigRepository.queryByCode("pwdErrorNumber");
                    int error = Integer.parseInt(errorNumber.getValue());
                    if (!StringUtils.hasText(value)) {
                        //密码操作失败
                        redisTemplate.opsForValue()
                                .set(RequestContext.Catalog.Tenant.name() + account, String.valueOf(1), 60,
                                        TimeUnit.SECONDS);
                        loginResult = new GeneralResult<>(false, loginResult.getMessage() + ",已操作【1】次");
                    } else if (Integer.parseInt(value) >= error) {
                        Result lockResult = tenantService.lockByAccount(account);
                        boolean success = lockResult.isSuccess();
                        redisTemplate.delete(RequestContext.Catalog.Tenant.name() + account);
                        if (success) {
                            loginResult = new GeneralResult<>(false, "账号或者密码错误已达【" + error + "】次，若租户存在将被冻结，请联系管理员解冻");
                        } else {
                            loginResult = new GeneralResult<>(false, "认证服务存在问题");
                        }
                    } else {
                        redisTemplate.boundValueOps(RequestContext.Catalog.Tenant.name() + account).increment(1L);
                        String num = redisTemplate.opsForValue().get(RequestContext.Catalog.Tenant.name() + account);
                        loginResult = new GeneralResult<>(false, loginResult.getMessage() + ",已操作【" + num + "】次");
                    }
                    return loginResult;
                } catch (Exception e) {
                    log.error("Get error message：", e);
                    return loginResult;
                }
            }
        }
        redisTemplate.delete(RequestContext.Catalog.Tenant.name() + account);
        return loginResult;
    }

    /**
     * 租户登出
     *
     * @param requestContext 上下文
     * @return 登出结果
     */
    @Operation(summary = "租户登出")
    @PostMapping("/tenants/logout")
    public Result tenantLogout(
            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.logout(requestContext.getTarget());
    }


    /**
     * 校验用户密码
     *
     * @param requestContext 上下文
     * @param password       密码
     * @return 校验结果
     */
    @Operation(summary = "校验用户密码")
    @GetMapping("/check/password")
    public GeneralResult checkPassword(@Value(Common.REQ_CONTEXT) RequestContext requestContext,
                                       @RequestParam String password
    ) {
        switch (requestContext.getCatalog()) {
            case Tenant:
                return tenantService.checkPassword(requestContext, password);
            case Manager:
            case User:
                return userService.checkPassword(requestContext, password);
            default:
        }
        return new GeneralResult(false, "校验用户密码异常");
    }

    /**
     * 获取key
     *
     * @param apiKey 安全密钥
     * @return key
     */
    @Operation(summary = "获取key")
    @GetMapping("/key")
    public GeneralResult<AccountSecurity> key(@RequestParam(value = "apiKey") String apiKey) {
        return tenantService.apiKey(apiKey);
    }

    /**
     * 获取登录账号信息详情
     *
     * @param requestContext 请求人
     * @return 账号信息详情
     */
    @Operation(summary = "获取登录账号信息详情")
    @GetMapping("/login/detail")
    public GeneralResult<Map<String, Object>> loginDetail(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Map<String, Object> resultMap = MapTools.simpleMap("accountCategory", requestContext.getCatalog());
        if (Objects.requireNonNull(requestContext.getCatalog()) == RequestContext.Catalog.Tenant) {
            resultMap.put(requestContext.getCatalog().name(),
                    tenantService.detail(requestContext.getTarget()).getData());
            GeneralResult<TenantAccount> accountDetail =
                    cosInternelService.tenantAccountDetail(requestContext.getTarget(), requestContext.getApiKey());
            resultMap.put("tenantAccount", accountDetail.isSuccess() ? accountDetail.getData() : null);
        } else {
            resultMap
                    .put(requestContext.getCatalog().name(), userService.detail(requestContext.getTarget()).getData());
        }
        Long projectId = Long.parseLong(
                Optional.ofNullable(redisTemplate.opsForValue().get("current_project_" + requestContext.getApiKey()))
                        .orElse("0"));
        resultMap.put("projectId", projectId);
        return new GeneralResult<>(true, resultMap, "获取登录账号信息成功");
    }

}
