package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.Business;
import com.bocloud.sms.entity.Project;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.ProjectService;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.ProjectModel;
import com.bocloud.sms.model.ProjectUserBean;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/projects")
@Tag(name = "项目关联")
@Slf4j
public class ProjectController {

    private final ProjectService projectService;
    private final UserService userService;
    private final StringRedisTemplate redisTemplate;


    public ProjectController(ProjectService projectService, UserService userService, StringRedisTemplate redisTemplate) {
        this.projectService = projectService;
        this.userService = userService;
        this.redisTemplate = redisTemplate;
    }

    /**
     * 查询项目列表
     *
     * @param pager          分页参数
     * @param requestContext 请求上下文
     * @return 项目列表
     */
    @Operation(summary = "获取项目列表")
    @GetMapping
    public GeneralResult<GridBean<Project>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                                                 @RequestParam(value = "tenantId", required = false, defaultValue = "false") Boolean tenantId) {
        return projectService.list(pager, requestContext, tenantId);
    }


    /**
     * 字典查询项目信息
     *
     * @param condition 字典值
     * @return 项目信息
     */
    @Operation(summary = "字典查询项目信息")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.condition(condition, requestContext);
    }

    /**
     * 创建项目
     *
     * @param projectModel   项目
     * @param requestContext 上下文
     * @return 创建结果
     */
    @Operation(summary = "创建项目")
    @PostMapping
    public Result create(@RequestBody ProjectModel projectModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.create(projectModel, requestContext);
    }

    /**
     * 切换项目
     *
     * @param id 项目id
     * @return 项目切换结果
     */
    @Operation(summary = "切换项目")
    @PatchMapping("/{id}")
    public Result switchProject(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        String apiKey = requestContext.getApiKey();
        Assert.isTrue(StringUtils.isNotEmpty(apiKey), "当前会话存在异常");
        try {
            redisTemplate.opsForValue().set("current_project_" + apiKey, String.valueOf(id));
            return new Result(true, "项目切换成功");
        } catch (Exception e) {
            return new Result(false, "项目切换失败");
        }
    }


    /**
     * 修改项目
     *
     * @param id             项目id
     * @param projectModel   项目
     * @param requestContext 上下文
     * @return 修改结果
     */
    @Operation(summary = "修改项目信息")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody ProjectModel projectModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.modify(id, projectModel, requestContext);
    }

    /**
     * 删除项目
     *
     * @param id             项目id
     * @param requestContext 上下文
     * @return 删除结果
     */
    @Operation(summary = "删除项目")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.remove(id, requestContext);
    }


    /**
     * 批量删除项目
     *
     * @param ids            项目id列表
     * @param requestContext 上下文
     * @return 批量删除结果
     */
    @Operation(summary = "批量删除项目")
    @DeleteMapping
    public Result batchRemove(@RequestBody List<Long> ids, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.batchRemove(ids, requestContext);
    }

    /**
     * 查看项目详情
     *
     * @param id 项目id
     * @return 项目
     */
    @Operation(summary = "查看项目详情")
    @GetMapping("/{id}")
    public GeneralResult<Project> detail(@PathVariable(value = Common.ID) Long id) {
        return projectService.detail(id);
    }

    /**
     * 查看项目所在业务
     *
     * @param id 项目id
     * @return 业务
     */
    @Operation(summary = "查看项目所在业务信息")
    @GetMapping("/{id}/business")
    public GeneralResult<Business> business(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return projectService.getBusinessDetail(id, requestContext);
    }

    /**
     * 项目配置用户
     *
     * @param id              项目id
     * @param projectUserBean 项目用户
     * @return 配置结果
     */
    @Operation(summary = "项目配追用户")
    @PostMapping("/{id}/users")
    public Result configUser(@PathVariable(value = Common.ID) Long id, @RequestBody ProjectUserBean projectUserBean) {
        return projectService.configUser(id, projectUserBean);
    }

    /**
     * 项目用户设置为项目经理
     *
     * @param id     项目id
     * @param userId 用户id
     * @return 设置结果
     */
    @Operation(summary = "项目用户设置为项目经理")
    @PatchMapping("/{id}/users/{userId}")
    public Result configUser(@PathVariable(value = Common.ID) Long id, @PathVariable(value = "userId") Long userId) {
        return projectService.configProjectManager(id, userId);
    }

    /**
     * 查询项目下关联的的用户ID
     *
     * @param id 项目id
     * @return 用户列表
     */
    @Operation(summary = "查询项目下关联用户")
    @GetMapping("/{id}/users")
    public GeneralResult<List<User>> listByProject(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return userService.listProjectUser(id, requestContext);
    }


    /**
     * 查询项目经理
     *
     * @param id 项目
     * @return 用户列表
     */
    @Operation(summary = "查询项目经理")
    @GetMapping("/{id}/managers")
    public GeneralResult<List<User>> listProjectManager(@PathVariable(value = Common.ID) Long id) {
        return userService.listProjectManager(id);
    }

    /**
     * 导出项目列表
     *
     * @param pager          分页参数
     * @param requestContext 请求上下文
     * @return 项目列表
     */
    @Deprecated
    @Operation(summary = "导出项目列表")
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                       HttpServletRequest request, HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        if (paramList.size() == 0) {
            paramList.add(new Param(MapTools.simpleMap("tenantId", 0), Sign.UEQ));
        }
        try {
            XSSFWorkbook workbook = projectService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "项目统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.name());
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            log.error("export project error,", e);
        }
    }

    /**
     * 下载项目列表模板
     *
     * @param request        浏览器请求
     * @param response       浏览器响应
     * @param requestContext 上下文
     */
    @Operation(summary = "下载项目列表模板")
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletRequest request, HttpServletResponse response,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        projectService.importTemp(response);
    }

    /**
     * 导入项目列表
     *
     * @param requestContext 上下文
     * @param request        浏览器请求
     * @return 导入结果
     */
    @Operation(summary = "导入项目列表")
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return projectService.importData(multipartFile, requestContext);
    }

    /**
     * 查询某组织下所有项目
     *
     * @param requestContext 请求上下文
     * @return 项目列表
     */
    @Operation(summary = "根据组织名称查询某组织下所有项目")
    @GetMapping("/by/depart")
    public GeneralResult<List<Project>> queryByDepart(@Value(Common.REQ_CONTEXT) RequestContext requestContext,
                                                 @RequestParam(value = "departName") String departName) {
        return projectService.queryByDepart(departName);
    }

    @Operation(summary = "根据组织ID查询某组织下所有项目")
    @GetMapping("/by/depart/ids")
    public GeneralResult<List<Project>> queryByDepart(Pager pager, @RequestParam(value = "departIds") List<Long> departIds) {
        return projectService.queryByDepartIds(pager, departIds);
    }

    @Operation(summary = "根据项目名称获取项目")
    @GetMapping("/by/name")
    public GeneralResult<Project> getByProjectName(@RequestParam(value = "projectName") String projectName) {
        return projectService.getByProjectName(projectName);
    }
}
