package com.bocloud.sms.service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.sms.entity.ApiToken;
import com.bocloud.sms.interfaces.ApiTokenService;
import com.bocloud.sms.repository.ApiTokenRepository;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.IDFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * api token访问控制业务实现
 */
@Service("apiTokenService")
@Slf4j
public class ApiTokenServiceImpl implements ApiTokenService {

    @Autowired
    private ApiTokenRepository apiTokenRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<ApiToken> get(String token) {
        try {
            ApiToken apiToken = apiTokenRepository.get(token);
            Assert.notNull(apiToken, "api token数据不存在");
            return new GeneralResult<>(true, apiToken, "查询token成功");
        } catch (Exception e) {
            log.error("query api token fail:", e);
            return new GeneralResult<>(false, "查询api token失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<ApiToken> create(ApiToken token, RequestContext context) {
        try {
            Assert.notNull(token.getApis(), "Api为空");
            token.setCreatorId(context.getTarget());
            token.setToken(IDFactory.instance().uuid());
            token.setGmtCreate(new Date());
            apiTokenRepository.save(token);
            if (token.getEnabled()) {
                redisTemplate.opsForValue().set("Api_Token_" + token.getToken(), token.getApis());
            }
            return new GeneralResult<>(true, "创建成功");
        } catch (DataBaseException e) {
            log.error("creat api token fail:", e);
            return new GeneralResult<>(false, "创建失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<GridBean<ApiToken>> list(Pager pager, RequestContext context) {
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            List<ApiToken> tokens = apiTokenRepository.list(pager);
            int total = apiTokenRepository.count(pager);
            GridBean<ApiToken> bean = new GridBean<>(pager.getPage(), pager.getRows(), total, tokens);
            return new GeneralResult<>(true, bean, "查询API列表成功");
        } catch (Exception e) {
            log.error("query api token fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> modify(ApiToken param, RequestContext context) {
        try {
            ApiToken token = apiTokenRepository.query(param.getId());
            Assert.notNull(token, "Api Token数据不存在");
            token.setApis(param.getApis());
            token.setRemark(param.getRemark());
            token.setMenderId(context.getTarget());
            token.setGmtModify(new Date());
            token.setEnabled(param.getEnabled());
            boolean result = apiTokenRepository.update(token);
            if (result) {
                if (token.getEnabled()) {
                    redisTemplate.opsForValue().set("Api_Token_" + token.getToken(), token.getApis());
                } else {
                    redisTemplate.delete("Api_Token_" + token.getToken());
                }
            }
            return new GeneralResult<>(result, result ? "修改成功" : "修改失败");
        } catch (DataBaseException e) {
            log.error("modify api token fail:", e);
            return new GeneralResult<>(false, "修改失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, RequestContext context) {
        try {
            ApiToken token = apiTokenRepository.query(id);
            if (null == token) {
                return new GeneralResult<>(false, "Api Token数据不存在");
            }
            boolean result = apiTokenRepository.delete(id, context.getTarget());
            if (result) {
                redisTemplate.delete("Api_Token_" + token.getToken());
            }
            return new GeneralResult<>(result, result ? "删除成功" : "删除失败");
        } catch (DataBaseException e) {
            log.error("remove api token fail:", e);
            return new GeneralResult<>(false, "删除失败:" + e.getMessage());
        }
    }

}
