package com.bocloud.sms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.sms.entity.Permission;
import com.bocloud.sms.entity.Plugin;
import com.bocloud.sms.entity.Project;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.PermissionService;
import com.bocloud.sms.interfaces.PluginService;
import com.bocloud.sms.interfaces.UserPermissionService;
import com.bocloud.sms.model.CloudServiceBean;
import com.bocloud.sms.model.PermissionBean;
import com.bocloud.sms.repository.CloudServiceBeanRepository;
import com.bocloud.sms.repository.PermissionRepository;
import com.bocloud.sms.repository.ProjectRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.utils.PermitComparator;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.MapTools;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 用户权限
 *
 * @author zhangyf
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class UserPermissionServiceImpl implements UserPermissionService {

    private final UserRepository userRepository;
    private final PermissionRepository permissionRepository;
    private final ProjectRepository projectRepository;
    private final PermissionService permissionService;
    private final CloudServiceBeanRepository cloudServiceBeanRepository;
    private final PluginService pluginService;

    /**
     * 菜单权限是否使用插件校验
     */
    @Value("${plugin.permission.enable:true}")
    private boolean validatePlugin;


    /**
     * 查询用户权限树形列表
     *
     * @param context
     * @return
     */
    @Override
    public GeneralResult<Map<String, Object>> listPermissions(RequestContext context) {
        try {
            //查询的是所属租户的菜单权限
            GeneralResult<List<PermissionBean>> permissionResult = getPermissionMap(context.getTarget());
            Assert.isTrue(permissionResult.isSuccess(), "获取用户菜单权限失败");
            Map<String, Object> resultMap = MapTools.simpleMap("permission", permissionResult.getData());
            resultMap.put("accountCategory", context.getCatalog());
            List<Project> projects = projectRepository.listUserManagerProjects(context.getTarget());
            resultMap.put("projectManager", projects.size() > 0);
            return new GeneralResult<>(true, resultMap, "获取用户所有权限成功");
        } catch (Exception e) {
            log.error("get user permission fail:", e);
            return new GeneralResult<>(false, "获取用户所有权限失败");
        }
    }

    @Override
    public List<Permission> listButtonPermissions(RequestContext context) {
        if (RequestContext.Catalog.Manager == context.getCatalog()) {
            return this.permissionRepository.listButtonPermissions(context);
        } else {
            return this.permissionRepository.listTenantButtonPermissions(context);
        }
    }

    /**
     * 获取用户服务菜单权限
     *
     * @param userId 用户ID
     * @return 用户菜单集合
     */
    private GeneralResult<List<PermissionBean>> getPermissionMap(Long userId) {
        User user = userRepository.query(userId);
        List<CloudServiceBean> cloudServiceBeanList = cloudServiceBeanRepository.getFavoriteServices(user.getTenantId(),
                RequestContext.Catalog.Tenant.name());
        Map<Long, CloudServiceBean> serviceMap = cloudServiceBeanList.stream()
                .collect(Collectors.toMap(CloudServiceBean::getId, Function.identity(), (m1, m2) -> m1));

        // 获取所有服务相关权限
        List<Long> serviceIds = serviceMap.keySet().stream().distinct().collect(Collectors.toList());
        GeneralResult<List<Permission>> permissionResult = permissionService.getServicePermissions(serviceIds);
        List<Permission> permissions = JSONArray.parseArray(JSONObject.toJSONString(permissionResult.getData()), Permission.class);
        List<Plugin> plugins = this.pluginService.runningPlugins(BoCloudService.CMP.getName(), BoCloudService.COS.getName(),
                BoCloudService.COP.getName());
        List<PermissionBean> servicePermissionBeans = permissions.stream().filter(permission -> {
                    if (StringUtils.isBlank(permission.getPluginsCode()) || plugins == null || !validatePlugin) {
                        return true;
                    }
                    return Optional.of(plugins).orElse(new ArrayList<>()).stream().anyMatch(plugin -> {
                        // 如果菜单绑定插件为空或者为开启状态，则进行添加操作
                        boolean pluginsStatus = permission.getPluginsStatus() != null && permission.getPluginsStatus();
                        return org.apache.commons.lang3.StringUtils.isBlank(permission.getPluginsCode()) ||
                                (pluginsStatus && permission.getPluginsCode().equals(plugin.getCode()));
                    });
                }).map(permit -> new PermissionBean(permit.getId(), permit.getName(), permit.getPath(), permit.getIcon(),
                        permit.getParentId(), permit.getPriority(), permit.getRouter(), permit.getComponent(),
                        permit.getRedirect(), permit.getMeta(), permit.getParams(),
                        permit.getCategory(), permit.getPluginsCode(), permit.getPluginsStatus(), permit.getButtonCode()))
                .collect(Collectors.toList());
        List<PermissionBean> servicePermissions = getPermission(0L, servicePermissionBeans);
        return new GeneralResult<>(true, servicePermissions, "获取服务菜单权限成功");
    }

    /**
     * 对用户权限进行递归排序
     *
     * @param pid
     * @param permissionBeans
     * @return
     */
    private List<PermissionBean> getPermission(Long pid, List<PermissionBean> permissionBeans) {
        return permissionBeans.stream()
                .filter(permissionBean -> null != permissionBean.getParentId() && permissionBean.getParentId().equals(pid))
                .peek(permissionBean -> {
                    List<PermissionBean> children = getPermission(permissionBean.getId(), permissionBeans);
                    if (children.isEmpty()) {
                        permissionBean.setChildren(null);
                    } else {
                        permissionBean.setChildren(children);
                    }
                }).sorted(new PermitComparator()).collect(Collectors.toList());
    }

}
