package com.bocloud.sms.service.upgrade;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.druid.pool.DruidDataSource;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.sms.entity.UpgradeHistory;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.UpgradeService;
import com.bocloud.sms.model.NotifyMessage;
import com.bocloud.sms.repository.UpgradeHistoryRepository;
import com.bocloud.sms.repository.UpgradeTaskRepository;
import com.bocloud.sms.repository.UserRepository;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.OperateResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 系统升级
 *
 * @author luyanchao
 * @date 2021-06-09
 */
@Service("upgradeService")
@Slf4j
public class UpgradeServiceImpl implements UpgradeService {

    private static final String LOCK = "SystemUpgrade";
    private static final String SUCCEED = "执行成功";
    private static final String INTERRUPTED = "执行中断";

    /**
     * 根据任务执行情况过滤接下来可以执行的任务
     *
     * @param allTasks
     *            所有要执行的任务
     * @param succeedTasks
     *            已经执行成功的任务
     * @param failedTasks
     *            已经执行失败的任务
     * @return 接下来可以执行的任务
     */
    private Map<String, UpgradeTask> getPreparedTasks(Map<String, UpgradeTask> allTasks,
        Map<String, UpgradeTask> succeedTasks, Map<String, UpgradeTask> failedTasks) {
        Map<String, UpgradeTask> preparedTasks = new HashMap<>();
        for (Map.Entry<String, UpgradeTask> thisTask : allTasks.entrySet()) {
            // 已经执行成功的任务无需再次执行
            if (succeedTasks.containsKey(thisTask.getKey())) {
                continue;
            }
            // 已经执行失败的任务无需再次执行
            if (failedTasks.containsKey(thisTask.getKey())) {
                continue;
            }
            // 依赖的任务有执行失败的，则此任务无法执行
            if (thisTask.getValue().getDependOnTasks().stream().anyMatch(failedTasks::containsValue)) {
                continue;
            }
            // 依赖的任务有尚未执行的，则此任务无法执行
            if (thisTask.getValue().getDependOnTasks().stream().anyMatch(task -> !succeedTasks.containsValue(task))) {
                continue;
            }
            // 剩下的要么没有执行过、要么没有依赖的任务、要么依赖的任务都执行成功了，那么此任务也就可以执行了
            preparedTasks.put(thisTask.getKey(), thisTask.getValue());
        }
        return preparedTasks;
    }

    private void sendMessageAndEmail(boolean success, String message, String emailTitle, String emailContent,
        RequestContext context) {
        OperateResult operateResult = new OperateResult(success, message, "system.upgrade", null,
            OperateResult.OperateCategory.ResourceEvent, context);
        NotifyMessage notifyMessage = new NotifyMessage(context, emailTitle, emailContent, emailContent, true, false);
        amqpTemplate.convertAndSend("event.notice.queue", notifyMessage);
        amqpTemplate.convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#",
            operateResult);
    }

    @Override
    public GeneralResult<Boolean> newMigration(String url, String user, String pass, RequestContext requestContext) {
        log.info("开始执行数据库迁移任务, 源数据库: {}", url);
        // 准备上一版本数据库的数据源
        DruidDataSource currDataSource = context.getBean(DruidDataSource.class);
        DruidDataSource oldDataSource = currDataSource.cloneDruidDataSource();
        oldDataSource.getConnectProperties().setProperty("config.decrypt", "false");
        oldDataSource.setUrl(url);
        oldDataSource.setUsername(user);
        oldDataSource.setPassword(new AESEncryptor().decrypt(pass, null));
        // 检查上一版本的数据源
        new JdbcTemplate(oldDataSource).queryForList("select 1");
        // 查找所有的UpgradeTask
        Map<String, UpgradeTask> upgradeTasks = context.getBeansOfType(UpgradeTask.class);
        // 为升级任务设置数据源
        upgradeTasks.values().forEach(task -> task.setDataSource(oldDataSource));
        log.info("本次数据库迁移任务数量：{}个", upgradeTasks.size());
        // UpgradeHistory数据入库
        final UpgradeHistory upgradeHistory = new UpgradeHistory();
        upgradeHistory.setUrl(url);
        upgradeHistory.setUser(user);
        upgradeHistory.setPass(pass);
        upgradeHistory.setTotal(upgradeTasks.size());
        upgradeHistory.setStatus(UpgradeStatus.WAITING.name());
        upgradeHistory.setComplete(0);
        upgradeHistory.setStartTime(new Date());
        upgradeHistory.setCreatorId(requestContext.getTarget());
        upgradeHistoryRepository.save(upgradeHistory);
        // UpgradeTask数据入库
        Map<String, com.bocloud.sms.entity.UpgradeTask> databaseTasks = new HashMap<>();
        for (Map.Entry<String, UpgradeTask> taskEntry : upgradeTasks.entrySet()) {
            UpgradeTask upgradeTask = taskEntry.getValue();
            String beanId = taskEntry.getKey();

            com.bocloud.sms.entity.UpgradeTask databaseTask = new com.bocloud.sms.entity.UpgradeTask();
            databaseTask.setUpgradeHistoryId(upgradeHistory.getId());
            databaseTask.setName(upgradeTask.getName());
            databaseTask.setStatus(UpgradeStatus.WAITING.name());
            databaseTask.setBeanId(beanId);
            databaseTask.setClassName(upgradeTask.getClass().getName());
            databaseTask.setCreatorId(requestContext.getTarget());
            upgradeTaskRepository.save(databaseTask);

            databaseTasks.put(beanId, databaseTask);
        }

        // 异步执行升级过程
        taskExecutor.execute(() -> {
            try (AutoCloseLock lock = lockFactory.getACLock(LOCK)) {
                Assert.isTrue(lock.acquire(5, TimeUnit.SECONDS), "当前已有数据迁移任务正在执行，请稍后再试");
                // 更新数据库
                upgradeHistory.setStatus(UpgradeStatus.RUNNING.name());
                upgradeHistoryRepository.update(upgradeHistory);
                // 准备任务数据
                Map<String, UpgradeTask> succeedTasks = new HashMap<>();
                Map<String, UpgradeTask> failedTasks = new HashMap<>();
                Map<String, UpgradeTask> preparedTasks = this.getPreparedTasks(upgradeTasks, succeedTasks, failedTasks);
                int index = 1;
                UpgradeException upgradeException = new UpgradeException();
                while (!preparedTasks.isEmpty()) {
                    log.info("开始执行第{}批数据迁移任务：{}", index,
                        preparedTasks.values().stream().map(UpgradeTask::getName).collect(Collectors.joining(", ")));
                    Map<String, Future> taskFutures = new HashMap<>();
                    // 开始执行任务并保留Future
                    for (Map.Entry<String, UpgradeTask> taskEntry : preparedTasks.entrySet()) {
                        UpgradeTask upgradeTask = taskEntry.getValue();
                        String beanId = taskEntry.getKey();
                        // 执行升级任务
                        Future<?> taskFuture = taskExecutor.submit(upgradeTask);
                        taskFutures.put(beanId, taskFuture);
                        // 更新数据库
                        com.bocloud.sms.entity.UpgradeTask databaseTask = databaseTasks.get(beanId);
                        databaseTask.setStartTime(new Date());
                        databaseTask.setStatus(UpgradeStatus.RUNNING.name());
                        upgradeTaskRepository.update(databaseTask);
                    }
                    // 获取Future执行结果
                    Map<String, Future> succeedFutures = new HashMap<>();
                    Map<String, Future> failedFutures = new HashMap<>();
                    while (succeedFutures.size() + failedFutures.size() < taskFutures.size()) {
                        for (Map.Entry<String, Future> futureEntry : taskFutures.entrySet()) {
                            Future taskFuture = futureEntry.getValue();
                            String beanId = futureEntry.getKey();
                            UpgradeTask upgradeTask = preparedTasks.get(beanId);
                            com.bocloud.sms.entity.UpgradeTask databaseTask = databaseTasks.get(beanId);
                            if (succeedFutures.containsKey(beanId) || failedFutures.containsKey(beanId)) {
                                continue;
                            }
                            if (taskFuture.isDone()) {
                                try {
                                    taskFuture.get();
                                    // 更新数据库
                                    this.setUpgradeTaskSucceed(databaseTask.getId());
                                    succeedTasks.put(beanId, upgradeTask);
                                    succeedFutures.put(beanId, taskFuture);
                                } catch (InterruptedException e) {
                                    // 更新数据库
                                    this.setUpgradeTaskInterrupted(databaseTask.getId());
                                    log.warn("任务[{}]被打断执行：[{}]", upgradeTask.getName(), e.getMessage());
                                    failedTasks.put(beanId, upgradeTask);
                                    failedFutures.put(beanId, taskFuture);
                                    upgradeException.addThrowable(upgradeTask.getName(), e);
                                    Thread.currentThread().interrupt();
                                } catch (ExecutionException e) {
                                    // 更新数据库
                                    Throwable t = Optional.of(e).map(ExecutionException::getCause).orElse(e);
                                    this.setUpgradeTaskFailed(databaseTask.getId(), t.getMessage());
                                    log.warn("任务[{}]执行失败：[{}]", upgradeTask.getName(), t.getMessage());
                                    failedTasks.put(beanId, upgradeTask);
                                    failedFutures.put(beanId, taskFuture);
                                    upgradeException.addThrowable(upgradeTask.getName(), t);
                                }
                            } else if (taskFuture.isCancelled()) {
                                this.setUpgradeTaskInterrupted(databaseTask.getId());
                                log.warn("任务[{}]被打断执行", upgradeTask.getName());
                                failedTasks.put(beanId, upgradeTask);
                                failedFutures.put(beanId, taskFuture);
                                upgradeException.addThrowable(upgradeTask.getName(), new InterruptedException());
                            } else {
                                log.debug("{}: {}%", upgradeTask.getName(), upgradeTask.getPercent());
                            }
                        }
                        Thread.sleep(1000);
                    }

                    preparedTasks = this.getPreparedTasks(upgradeTasks, succeedTasks, failedTasks);
                    index++;
                }
                if (upgradeException.hasThrowable()) {
                    throw upgradeException;
                }
                // 更新数据库
                this.setUpgradeHistorySucceed(upgradeHistory.getId());
                // 发消息和站内信
                String message = String.format("您好，从数据库%s迁移数据任务执行成功！", url);
                this.sendMessageAndEmail(true, message, "数据迁移成功", message, requestContext);
                log.info("从数据库{}迁移数据任务执行成功", url);
            } catch (Exception e) {
                log.error("从数据库{}迁移数据任务执行失败:{}", url, e.getMessage(), e);
                // 更新数据库
                this.setUpgradeHistoryFailed(upgradeHistory.getId(), e.getMessage());
                // 发消息和站内信
                String message = String.format("您好，从数据库%s迁移数据任务执行失败，异常信息：%s。请重新创建当前版本标准库后再试", url, e.getMessage());
                this.sendMessageAndEmail(false, message, "数据迁移失败", message, requestContext);
                Thread.currentThread().interrupt();
            }
        });
        return new GeneralResult<>(true, true, "任务已经下发，正在执行...");
    }

    /**
     * 更新升级记录为失败状态
     *
     * @param id
     *            升级记录id
     * @param error
     *            异常信息
     */
    private void setUpgradeHistoryFailed(Long id, String error) {
        UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(id);
        upgradeHistory.setEndTime(new Date());
        upgradeHistory.setStatus(UpgradeStatus.FAILED.name());
        upgradeHistory.setResult(error);
        upgradeHistoryRepository.update(upgradeHistory);
    }

    /**
     * 更新升级记录为成功状态
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeHistorySucceed(Long id) {
        UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(id);
        upgradeHistory.setEndTime(new Date());
        upgradeHistory.setStatus(UpgradeStatus.SUCCEED.name());
        upgradeHistory.setResult(SUCCEED);
        upgradeHistoryRepository.update(upgradeHistory);
    }

    /**
     * 更新升级记录为中断状态
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeHistoryInterruped(Long id) {
        UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(id);
        upgradeHistory.setEndTime(new Date());
        upgradeHistory.setStatus(UpgradeStatus.INTERRUPTED.name());
        upgradeHistory.setResult(INTERRUPTED);
        upgradeHistoryRepository.update(upgradeHistory);
    }

    /**
     * 更新升级记录为运行状态
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeHistoryRunning(Long id) {
        UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(id);
        upgradeHistory.setStatus(UpgradeStatus.RUNNING.name());
        upgradeHistory.setResult(null);
        upgradeHistory.setEndTime(null);
        upgradeHistoryRepository.update(upgradeHistory);
    }

    /**
     * 更新升级任务为运行状态
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeTaskRunning(Long id) {
        com.bocloud.sms.entity.UpgradeTask upgradeTask = upgradeTaskRepository.get(id);
        upgradeTask.setStatus(UpgradeStatus.RUNNING.name());
        upgradeTask.setResult(null);
        upgradeTask.setEndTime(null);
        upgradeTaskRepository.update(upgradeTask);
    }

    /**
     * 更新升级任务为成功状态
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeTaskSucceed(Long id) {
        com.bocloud.sms.entity.UpgradeTask upgradeTask = upgradeTaskRepository.get(id);
        upgradeTask.setStatus(UpgradeStatus.SUCCEED.name());
        upgradeTask.setResult(SUCCEED);
        upgradeTask.setEndTime(new Date());
        upgradeTaskRepository.update(upgradeTask);
    }

    /**
     * 更新升级任务为失败状态
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeTaskFailed(Long id, String error) {
        com.bocloud.sms.entity.UpgradeTask upgradeTask = upgradeTaskRepository.get(id);
        upgradeTask.setStatus(UpgradeStatus.FAILED.name());
        upgradeTask.setResult(error);
        upgradeTask.setEndTime(new Date());
        upgradeTaskRepository.update(upgradeTask);
    }

    /**
     * 更新升级任务为打断
     *
     * @param id
     *            升级记录id
     */
    private void setUpgradeTaskInterrupted(Long id) {
        com.bocloud.sms.entity.UpgradeTask upgradeTask = upgradeTaskRepository.get(id);
        upgradeTask.setStatus(UpgradeStatus.INTERRUPTED.name());
        upgradeTask.setResult(INTERRUPTED);
        upgradeTask.setEndTime(new Date());
        upgradeTaskRepository.update(upgradeTask);
    }

    @Override
    public GeneralResult<Boolean> retryMigration(Long id, RequestContext requestContext) {
        UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(id);
        Assert.notNull(upgradeHistory, "未找到id为" + id + "的历史迁移任务");
        return this.newMigration(upgradeHistory.getUrl(), upgradeHistory.getUser(), upgradeHistory.getPass(),
            requestContext);
    }

    @Override
    public GeneralResult<GridBean<UpgradeHistory>> listUpgradeHistory(Pager pager) {
        List<UpgradeHistory> list = upgradeHistoryRepository.list(pager);
        for (UpgradeHistory item : list) {
            List<com.bocloud.sms.entity.UpgradeTask> tasks = upgradeTaskRepository.list(item.getId(),
                new Pager(0, Integer.MAX_VALUE, new ArrayList<>(), new HashMap<>()));
            item.setTasks(tasks);
            item.setTotal(tasks.size());
            item.setComplete((int)tasks.stream().map(com.bocloud.sms.entity.UpgradeTask::getStatus)
                .filter(UpgradeStatus.SUCCEED.name()::equals).count());
            User creator = userRepository.query(item.getCreatorId());
            item.setCreator(creator.getName());
        }
        int count = upgradeHistoryRepository.count(pager.getParams());
        GridBean<UpgradeHistory> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), count, list);
        return new GeneralResult<>(true, gridBean, "执行成功");
    }

    @Override
    public GeneralResult<UpgradeHistory> getUpgradeHistory(Long id) {
        UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(id);
        return new GeneralResult<>(true, upgradeHistory, "执行成功");
    }

    @Override
    public GeneralResult<GridBean<com.bocloud.sms.entity.UpgradeTask>> listUpgradeTask(Long upgradeHistoryId,
        Pager pager) {
        List<com.bocloud.sms.entity.UpgradeTask> list = upgradeTaskRepository.list(upgradeHistoryId, pager);
        // 如果这个迁移任务正在执行，则获取各个子任务的进度
        UpgradeHistory upgrade = upgradeHistoryRepository.get(upgradeHistoryId);
        for (com.bocloud.sms.entity.UpgradeTask item : list) {
            if (item.getStatus().equals(UpgradeStatus.RUNNING.name())) {
                UpgradeTask task = context.getBean(item.getBeanId(), UpgradeTask.class);
                item.setPercent(task.getPercent());
            } else if (item.getStatus().equals(UpgradeStatus.SUCCEED.name())) {
                item.setPercent(100);
            } else {
                item.setPercent(0);
            }
        }
        int count = upgradeTaskRepository.count(upgradeHistoryId, pager.getParams());
        GridBean<com.bocloud.sms.entity.UpgradeTask> gridBean =
            new GridBean<>(pager.getPage(), pager.getRows(), count, list);
        return new GeneralResult<>(true, gridBean, "执行成功");
    }

    @Override
    public GeneralResult<Boolean> retryUpgradeTask(Long taskId, RequestContext requestContext) {
        // 数据校验
        com.bocloud.sms.entity.UpgradeTask databaseTask = upgradeTaskRepository.get(taskId);
        Assert.notNull(databaseTask, "未找到id为" + taskId + "的历史迁移任务");
        final UpgradeHistory upgradeHistory = upgradeHistoryRepository.get(databaseTask.getUpgradeHistoryId());
        Assert.notNull(upgradeHistory, "未找到id为" + databaseTask.getUpgradeHistoryId() + "的历史迁移任务");
        final UpgradeTask currentTask = context.getBean(databaseTask.getBeanId(), UpgradeTask.class);
        // 查询所有任务
        List<com.bocloud.sms.entity.UpgradeTask> databaseTasks = upgradeTaskRepository.list(
            databaseTask.getUpgradeHistoryId(), new Pager(1, Integer.MAX_VALUE, new ArrayList<>(), new HashMap<>()));
        // 查找已经完成的UpgradeTask
        List<UpgradeTask> preparedTasks =
            databaseTasks.stream().filter(task -> task.getStatus().equals(UpgradeStatus.SUCCEED.name()))
                .map(com.bocloud.sms.entity.UpgradeTask::getBeanId)
                .map(beanId -> context.getBean(beanId, UpgradeTask.class)).collect(Collectors.toList());
        // 如果当前任务包含前置任务，则需要判断当前任务的前置任务是否已经完成
        if (!ListTool.isEmpty(currentTask.getDependOnTasks())) {
            List<UpgradeTask> unsatisfiedTasks = currentTask.getDependOnTasks().stream()
                .filter(task -> !preparedTasks.contains(task)).collect(Collectors.toList());
            Assert.isTrue(unsatisfiedTasks.isEmpty(), String.format("当前任务的前置任务[%s]尚未完成，请先执行前置任务",
                unsatisfiedTasks.stream().map(UpgradeTask::getName).collect(Collectors.joining("、"))));
        }
        // 准备上一版本数据库的数据源
        DruidDataSource currDataSource = context.getBean(DruidDataSource.class);
        DruidDataSource oldDataSource = currDataSource.cloneDruidDataSource();
        oldDataSource.getConnectProperties().setProperty("config.decrypt", "false");
        oldDataSource.setUrl(upgradeHistory.getUrl());
        oldDataSource.setUsername(upgradeHistory.getUser());
        oldDataSource.setPassword(new AESEncryptor().decrypt(upgradeHistory.getPass(), null));
        // 检查上一版本的数据源
        new JdbcTemplate(oldDataSource).queryForList("select 1");
        // 为当前任务设置数据源
        currentTask.setDataSource(oldDataSource);
        // 异步执行任务
        taskExecutor.execute(() -> {
            try (AutoCloseLock lock = lockFactory.getACLock(LOCK)) {
                Assert.isTrue(lock.acquire(5, TimeUnit.SECONDS), "当前已有数据迁移任务正在执行，请稍后再试");
                // 更新数据库
                this.setUpgradeHistoryRunning(upgradeHistory.getId());
                this.setUpgradeTaskRunning(databaseTask.getId());
                // 执行任务
                currentTask.run();
                // 更新数据库
                if (preparedTasks.size() + 1 == databaseTasks.size()) {
                    this.setUpgradeHistorySucceed(upgradeHistory.getId());
                } else {
                    this.setUpgradeHistoryInterruped(upgradeHistory.getId());
                }
                this.setUpgradeTaskSucceed(databaseTask.getId());
                // 发消息和站内信
                String message = String.format("您好，数据迁移任务%s任务执行成功！", currentTask.getName());
                this.sendMessageAndEmail(true, message, "数据迁移成功", message, requestContext);
            } catch (Exception e) {
                log.error("数据迁移任务{}任务执行:{}", currentTask.getName(), e.getMessage(), e);
                // 更新数据库
                this.setUpgradeHistoryFailed(upgradeHistory.getId(), e.getMessage());
                this.setUpgradeTaskFailed(databaseTask.getId(), e.getMessage());
                // 发消息和站内信
                String message = String.format("您好，数据迁移任务%s任务执行失败，异常信息：%s", currentTask.getName(), e.getMessage());
                this.sendMessageAndEmail(false, message, "数据迁移成功", message, requestContext);
            } finally {
                oldDataSource.close();
            }
        });
        return new GeneralResult<>(true, true, "任务已经下发，正在执行...");
    }

    @Autowired
    private ApplicationContext context;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private UpgradeHistoryRepository upgradeHistoryRepository;
    @Autowired
    private UpgradeTaskRepository upgradeTaskRepository;
    @Autowired
    private UserRepository userRepository;
}
