package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.megatron.framework.registry.RegistryClient;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiGettokenRequest;
import com.dingtalk.api.request.OapiMessageCorpconversationAsyncsendV2Request;
import com.dingtalk.api.request.OapiMessageCorpconversationGetsendresultRequest;
import com.dingtalk.api.response.OapiGettokenResponse;
import com.dingtalk.api.response.OapiMessageCorpconversationAsyncsendV2Response;
import com.dingtalk.api.response.OapiMessageCorpconversationGetsendresultResponse;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.model.Result;
import com.megatron.common.utils.JSONTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.Map;
import java.util.concurrent.CompletableFuture;

/**
 * @author lixin
 */
@Component("dingTalkSender")
@Slf4j
public class DingTalkSender {

    private static final String DING_TALK_PREFIX = "https://oapi.dingtalk.com";
    private static final String GET_TOKEN_URL = "/gettoken";
    private static final String SEND_MSG_URL = "/topapi/message/corpconversation/asyncsend_v2";
    private static final String GET_SEND_RESULT_URL = "/topapi/message/corpconversation/getsendresult";
    private static String accessToken = "";
    private static Long expireTime = 0L;
    private final RegistryClient client;
    private String appKey = "";
    private String appSecret = "";
    private Long agentId = 0L;

    public DingTalkSender(final RegistryClient client) {
        this.client = client;
    }

    /**
     * 获取token
     */
    public synchronized String getAccessToken(boolean flushNow) throws Exception {
        this.setConfig();
        if (System.currentTimeMillis() >= expireTime || flushNow) {
            DingTalkClient client1 = new DefaultDingTalkClient(DING_TALK_PREFIX + GET_TOKEN_URL);
            OapiGettokenRequest req1 = new OapiGettokenRequest();
            req1.setAppkey(appKey);
            req1.setAppsecret(appSecret);
            req1.setHttpMethod("GET");
            OapiGettokenResponse rsp = null;
            rsp = client1.execute(req1);
            if (rsp.isSuccess()) {
                accessToken = rsp.getAccessToken();
                //提前五分钟过期
                expireTime = (rsp.getExpiresIn() - 300) * 1000 + System.currentTimeMillis();
                log.info("get dingTalk access token success!access token:{}", rsp.getAccessToken());
            } else {
                throw new Exception(
                        "get dingTalk access token error! errorCode=" + rsp.getErrcode() + " errorMsg=" + rsp.getErrmsg());
            }
        }
        return accessToken;
    }

    /**
     * 以工作通知形式发送消息
     * 1. 同一个应用相同消息的内容同一个用户一天只能接收一次。
     * 2. 同一个应用给同一个用户发送消息，企业内部应用一天不得超过500次。
     * 3. 通过设置to_all_user参数全员推送消息，一天最多3次。
     * 该接口是异步发送消息，接口返回成功并不表示用户一定会收到消息，需要通过获取工作通知消息的发送结果接口查询是否给用户发送成功。
     *
     * @param content    发送内容
     * @param userIdList 用户ID集合
     * @param deptIdList 组织机构ID集合
     * @param isAll      全部发送
     */
    public Result sendMessage(String content, String userIdList, String deptIdList, boolean isAll) {
        try {
            boolean flag = false;
            if (!StringUtils.hasText(appKey) || !StringUtils.hasText(appSecret) || agentId == 0L) {
                //首次調用
                flag = true;
                setConfig();
            }
            Assert.hasText(content, "内容content不能为空");
            Assert.hasText(userIdList, "接收人userId不能为空");
            DingTalkClient client = new DefaultDingTalkClient(DING_TALK_PREFIX + SEND_MSG_URL);
            OapiMessageCorpconversationAsyncsendV2Request req = new OapiMessageCorpconversationAsyncsendV2Request();
            req.setAgentId(agentId);
            //指定 userid列表 长度100
            if (StringUtils.hasText(userIdList)) {
                req.setUseridList(userIdList);
            }
            //指定接收者组织机构id列表 长度20
            if (StringUtils.hasText(deptIdList)) {
                req.setDeptIdList(deptIdList);
            }
            // 是否全体通知
            req.setToAllUser(isAll);

            OapiMessageCorpconversationAsyncsendV2Request.Msg obj1 =
                    new OapiMessageCorpconversationAsyncsendV2Request.Msg();
            obj1.setMsgtype("text");
            OapiMessageCorpconversationAsyncsendV2Request.Text obj2 =
                    new OapiMessageCorpconversationAsyncsendV2Request.Text();
            obj2.setContent(content);
            obj1.setText(obj2);
            req.setMsg(obj1);
            OapiMessageCorpconversationAsyncsendV2Response rsp = client.execute(req, getAccessToken(flag));
            //取出task_id,同于查看任务发送结果/进度
            if (rsp.isSuccess()) {
                JSONObject jsonObject = JSONObject.parseObject(rsp.getBody());
                Long taskId = jsonObject.getLong("task_id");
                log.info("send dingTalk message end!taskId:{},userid:{},content:{}", taskId, userIdList, content);
                //后查询结果
                CompletableFuture.runAsync(() -> {
                    Map<String, JSONArray> result = getSendResult(taskId);
                    while (result.size() == 0) {
                        result = getSendResult(taskId);
                    }
                    Assert.isTrue(result != null, "请稍后查询本次任务执行结果,taskId:" + taskId);
                    if (result.get("read_user_id_list").isEmpty() && result.get("unread_user_id_list").isEmpty()) {
                        log.error("send dingTalk message failed: taskId=" + taskId);
                    } else {
                        log.info("send dingTalk message success: taskId=" + taskId);
                    }
                    //打印结果日志
                    result.forEach((k, v) -> {
                        if (!v.isEmpty()) {
                            //限流提示
                            if ("forbidden_list".equals(k)) {
                                for (Object o : v) {
                                    JSONObject object = (JSONObject) o;
                                    String code = object.getString("code");
                                    switch (code) {
                                        case "143106":
                                            object.put("msg", "单应用给单人推送重复消息超限");
                                            object.put("solution", "给同一员工一天只能发送一条内容相同的消息通知");
                                            break;
                                        case "143105":
                                            object.put("msg", "单应用给单人每日推送量超限");
                                            object.put("solution", "企业每天给每个员工最多可发送500条消息通知");
                                            break;
                                        default:
                                    }
                                }
                            }
                            log.info("=========" + k + ":" + v.toString());
                        }
                    });
                });
                return new Result(true, "");
            } else {
                log.error("send dingTalk message error!userid:{},content:{},errorCode:{},errorMsg:{}", userIdList,
                        content, rsp.getErrorCode(), rsp.getErrmsg());
            }
        } catch (Exception e) {
            log.error("send dingTalk message exception", e);
        }
        return new Result(false, "");
    }

    private void setConfig() throws Exception {
        String appKeyNow = "";
        String appSecretNow = "";
        String agentIdNow = "";
        String data = this.client.getSystem();
        JSONObject object = JSONTools.isJSONObj(data);
        if (null != object) {
            appKeyNow = object.getString("DingTalkAppKey");
            appSecretNow = object.getString("DingTalkAppSecret");
            agentIdNow = object.getString("DingTalkAgentId");
            if (!StringUtils.hasText(appKeyNow)) {
                throw new Exception("get system config--DingTalkAppKey is null!");
            }
            if (!StringUtils.hasText(appSecretNow)) {
                throw new Exception("get system config--DingTalkAppSecret is null!");
            }
            if (!StringUtils.hasText(agentIdNow)) {
                throw new Exception("get system config--DingTalkAgentId is null!");
            }
            log.info("set dingtalk config success!");
            this.appKey = appKeyNow;
            this.appSecret = new DESEncryptor().decrypt(appSecretNow, "BocloudCMP");
            this.agentId = Long.valueOf(agentIdNow);
        } else {
            throw new Exception("get system config is null");
        }

    }

    /**
     * 根据taskId查看异步任务执行结果
     *
     * @param taskId 任务ID
     * @return 任务执行结果
     */
    public Map<String, JSONArray> getSendResult(Long taskId) {
        Map<String, JSONArray> map = null;
        try {
            DingTalkClient client = new DefaultDingTalkClient(DING_TALK_PREFIX + GET_SEND_RESULT_URL);
            OapiMessageCorpconversationGetsendresultRequest req = new OapiMessageCorpconversationGetsendresultRequest();
            req.setAgentId(this.agentId);
            req.setTaskId(taskId);
            OapiMessageCorpconversationGetsendresultResponse rsp = client.execute(req, getAccessToken(false));
            Long errcode = rsp.getErrcode();
            if (errcode == 42001L || errcode == 40014L) {
                //token过期/不合法
                rsp = client.execute(req, getAccessToken(true));
            }
            if (rsp.isSuccess()) {
                JSONObject jsonObject = JSONObject.parseObject(rsp.getBody());
                map = (Map<String, JSONArray>) jsonObject.get("send_result");
            } else {
                log.error("get dingTalk task:{} result error!" + rsp.getErrmsg(), taskId);
            }
        } catch (Exception e) {
            log.error("get dingTalk task:" + taskId + " result exception:", e);
        }
        return map;
    }

}
