package com.bocloud.sms.repository;

import com.bocloud.cmp.boot.model.BocloudStatus;
import com.bocloud.sms.entity.Tenant;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/2/3 周一 11:38
 */

@Repository("tenantRepository")
public class TenantRepository extends JdbcGenericDao<Tenant, Long> {

    public TenantRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 删除租户
     *
     * @param id
     * @return
     */
    public boolean delete(Long id, Long menderId) {
        String sql = "update tenant set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 锁定租户
     *
     * @param id
     * @param menderId
     * @return
     */
    public boolean lock(Long id, Long menderId) {
        return handleStatus(id, menderId, BaseStatus.ABNORMAL);
    }

    /**
     * 批量冻结租户
     *
     * @param ids
     * @param menderId
     * @return
     */
    public boolean batchLock(List<Long> ids, Long menderId) {
        String sql = "update tenant set status = 'ABNORMAL' , gmt_modify = :gmtModify, mender_id = :menderId  where id in (:ids) and status!= 'LOGOUT' and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 激活租户
     *
     * @param id
     * @param menderId
     * @return
     */
    public boolean active(Long id, Long menderId) {
        return handleStatus(id, menderId, BaseStatus.NORMAL);
    }

    /**
     * 批量解冻租户
     *
     * @param ids
     * @param menderId
     * @return
     */
    public boolean batchActive(List<Long> ids, Long menderId) {
        String sql = "update tenant set status = 'NORMAL' , gmt_modify = :gmtModify, mender_id = :menderId  where id in (:ids) and status!= 'LOGOUT'  and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 注销租户
     *
     * @param id
     * @param menderId
     * @return
     */
    public boolean logout(Long id, Long menderId) {
        String sql = "update tenant set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", BocloudStatus.Basic.LOGOUT.name());
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    private boolean handleStatus(Long id, Long menderId, BaseStatus status) {
        String sql = "update tenant set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("status", status.name());
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据ID查询租户
     *
     * @param id
     * @return
     */
    public Tenant query(Long id) {
        String sql = "select a.* from tenant a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Tenant> list = this.list(Tenant.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one tenant in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询所有租户
     *
     * @return
     */
    public List<Tenant> list() {
        String sql = "select a.* from tenant a  where a.is_deleted = 0";
        return this.list(Tenant.class, sql);
    }

    /**
     * 查询所有租户数据
     *
     * @return
     */
    public List<Tenant> listAll() {
        String sql = "select * from tenant ";
        return this.list(Tenant.class, sql);
    }

    /**
     * 查询有过期时间的租户
     *
     * @return
     */
    public List<Tenant> listManagerHasExpiredDate() {
        String sql = "select * from tenant where is_deleted = 0 and expired_date is not null";
        return this.list(Tenant.class, sql);
    }

    /**
     * 根据城市编码获取租户
     *
     * @param city
     * @return
     */
    public List<Tenant> listByCity(String city) {
        String sql = "select a.* from tenant a where a.is_deleted = 0 and a.city=:city";
        return this.list(Tenant.class, sql, MapTools.simpleMap("city", city));
    }

    /**
     * 分页查询租户
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     */
    public List<Tenant> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from tenant a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.list(Tenant.class, sql, param);
    }

    /**
     * 简易查询租户
     *
     * @param params
     * @param sorter
     * @return
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name from tenant a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(Tenant.class, sql, paramMap).stream()
                .map(object -> new SimpleBean(object.getId(), object.getName())).collect(Collectors.toList());
    }

    public List<Tenant> listChildren(Long id) {
        String sql = "select a.* from tenant a where a.is_deleted = 0 and  a.parent_id = :id";
        Map<String, Object> paramsMap = MapTools.simpleMap("id", id);
        return this.list(Tenant.class, sql, paramsMap);
    }

    /**
     * 获取一级租户
     *
     * @return
     */
    public List<Tenant> listParentTenants() {
        String sql = "select a.* from tenant a where a.is_deleted = 0 and  a.parent_id = 0";
        return this.list(Tenant.class, sql, null);
    }

    /**
     * 获取有子租户的父租户
     *
     * @return
     */
    public List<Tenant> listParentHasChildren() {
        String sql = "select distinct a.* from tenant a left join tenant b on a.id = b.parent_id where a.is_deleted =0 and  b.is_deleted =0";
        return this.list(Tenant.class, sql);
    }

    /**
     * 获取租户数量
     *
     * @param params
     * @return
     */
    public int count(List<Param> params) {
        String sql = "select count(1) from tenant t where t.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "t");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    /**
     * 根据编码查询租户
     *
     * @param code
     * @return
     */
    public Tenant getByCode(String code) {
        String sql = "select a.* from tenant a where a.group_code = :code and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<Tenant> list = this.list(Tenant.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one tenant in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据账号查询租户
     *
     * @param account
     * @return
     */
    public Tenant getByAccount(String account) {
        String sql = "select a.* from tenant a where a.account = :account and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("account", account);
        List<Tenant> list = this.list(Tenant.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one tenant in table");
        return list.isEmpty() ? null : list.get(0);
    }


    /**
     * 根据名称查询租户
     *
     * @param name
     * @return
     */
    public Tenant getByName(String name) {
        String sql = "select a.* from tenant a where a.name = :name and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<Tenant> list = this.list(Tenant.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one tenant in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据邮箱查询租户
     *
     * @param email
     * @return
     */
    public Tenant getByEmail(String email) {
        String sql = "select a.* from tenant a where a.email = :email and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("email", email);
        List<Tenant> list = this.list(Tenant.class, sql, params);
        Assert.isTrue(list.size() <= 1, "more than one tenant in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 租户登录信息总数
     *
     * @param params
     * @return
     */
    public int loginTimes(List<Param> params) {
        String sql = "select count(1) from login_record where category = :category ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        Map<String, Object> paramsMap = MapTools.simpleMap("category", RequestContext.Catalog.Tenant.name());
        return Integer.parseInt(super.countQuery(sql, paramsMap).toString());
    }

    /**
     * 修改收藏夹排序顺序
     *
     * @param serviceId
     * @param priority
     * @param tenantId
     * @return
     */
    public boolean changePriority(Long serviceId, Integer priority, Long tenantId) {
        String sql = "update favorite set priority = :priority where service_id=:serviceId and target=:tenantId and category='Tenant' ";
        Map<String, Object> paramsMap = MapTools.simpleMap("serviceId", serviceId);
        paramsMap.put("priority", priority);
        paramsMap.put("tenantId", tenantId);
        return super.execute(sql, paramsMap) > 0;
    }

    public Integer count(String start, String end, String city) {
        String sql = "select count(a.id) from tenant a where a.is_deleted = 0 and a.city = :city";
        Map<String, Object> param = MapTools.simpleMap("city", city);
        if (StringUtils.hasText(start)) {
            sql += " and a.gmt_create >= :start";
            param.put("start", start);
        }
        if (StringUtils.hasText(end)) {
            sql += " and a.gmt_create <= :end";
            param.put("end", end);
        }
        return this.countQuery(sql, param).intValue();
    }
}
